%% Problem (Rosenbrock)
fun = @(x)100*(x(2) - x(1)^2)^2 + (1 - x(1))^2;

dfun = @(x)[-400*x(1)*(x(2) - x(1)^2) - 2*(1 - x(1)); 200*(x(2) - x(1)^2)];

ddfun =  @(x)[800*x(1)^2 - 400*(x(2) - x(1)^2) + 2, -400*x(1); ...
              -400*x(1), 200];

%%
x0 = unifrnd(-5, 5, 2, 1);
maxiter = 500; 
I = eye(length(x0));

%% Gradyant Inis (GI)

iter = 1; x = x0; 
gi_fgecmis = zeros(1, maxiter);
gi_fgecmis(iter) = fun(x);
dx = dfun(x); 
hata = norm(dx);

p = -dx; 

fprintf('\n\n%5s %15s %15s\n', 'adim', 'f degeri', 'df normu');
fprintf('======================================\n');
fprintf('%5d %15.4f %15.4f\n', iter, gi_fgecmis(iter), hata);
while (iter < maxiter)
    
    if (hata < 1.0e-4)
        break;
    end
    
    iter = iter + 1;
    
    [alpha, gi_fgecmis(iter)] = fminbnd(@(alpha)fun(x + alpha*p), 0, 1);

    x = x + alpha*p;
    
    dx = dfun(x);
    hata = norm(dx);
    
    p = -dx;
   
    fprintf('%5d %15.4f %15.4f\n', iter, gi_fgecmis(iter), hata);
    
 
end

fprintf('\nToplam adim sayisi: %d\nGradyant inis cozumu: (%.2f, %.2f)\nGradyant normu: %.4f\n', iter, x(1), x(2), hata);


%% Newton (N)

iter = 1; x = x0; 
n_fgecmis = zeros(1, maxiter);
n_fgecmis(iter) = fun(x);
dx = dfun(x); 
hata = norm(dx);

p = -(ddfun(x)\dx); 

fprintf('\n\n%5s %15s %15s\n', 'adim', 'f degeri', 'df normu');
fprintf('======================================\n');
fprintf('%5d %15.4f %15.4f\n', iter, n_fgecmis(iter), hata);
while (iter < maxiter)

    if (hata < 1.0e-4)
        break;
    end
    
    iter = iter + 1;
    
    [alpha, n_fgecmis(iter)] = fminbnd(@(alpha)fun(x + alpha*p), 0, 1);
    
    x = x + alpha*p;
    
    dx = dfun(x);
    hata = norm(dx);
        
    p = -(ddfun(x)\dx);
    
    fprintf('%5d %15.4f %15.4f\n', iter, n_fgecmis(iter), hata);
    
end

fprintf('\nToplam adim sayisi: %d\nNewton cozumu: (%.2f, %.2f)\nGradyant normu: %.4f\n', iter, x(1), x(2), hata);


%% Newton-benzeri (NB)

iter = 1; x = x0; 
nb_fgecmis = zeros(1, maxiter);
nb_fgecmis(iter) = fun(x);
dx = dfun(x); 
hata = norm(dx);
Binv = I;

p = -Binv*dx; 

fprintf('\n\n%5s %15s %15s\n', 'adim', 'f degeri', 'df normu');
fprintf('======================================\n');
fprintf('%5d %15.4f %15.4f\n', iter, nb_fgecmis(iter), hata);
while (iter < maxiter)

    if (hata < 1.0e-4)
        break;
    end
    
    iter = iter + 1;
    
    [alpha, nb_fgecmis(iter)] = fminbnd(@(alpha)fun(x + alpha*p), 0, 1);
    
    x = x + alpha*p;
    
    s = alpha*p;
    yold = dx;
    dx = dfun(x);
    hata = norm(dx);
    y = dx - yold;
    if (abs(y'*s) <= 1.0e-4^2)
        Binv = I;
    else
        Binv = (I - (s*y')/(y'*s))*Binv*(I - (y*s')/(y'*s)) + (s*s')/(y'*s);
    end
    
    p = -Binv*dx;
    
    fprintf('%5d %15.4f %15.4f\n', iter, nb_fgecmis(iter), hata);
    
end

fprintf('\nToplam adim sayisi: %d\nNewton-benzeri cozumu: (%.2f, %.2f)\nGradyant normu: %.4f\n', iter, x(1), x(2), hata);


%% Grafikler

plot(2:10, gi_fgecmis(2:10), 'b-', 'LineWidth', 3); hold on;
plot(2:10, n_fgecmis(2:10), 'g-', 'LineWidth', 3);
plot(2:10, nb_fgecmis(2:10), 'r-', 'LineWidth', 3); hold off;
legend('GI', 'N', 'NB'); grid on;
xlabel('Adimlar');
ylabel('Fonksiyon Degeri');
